#include "Board.h"


//=================================================================================================
// Board methods:


// Move robot 'id' relatively.
//
void Board::move(int id, int delta_x, int delta_y)
{
    int     x = roboinfo[id].pos.fst + delta_x;
    int     y = roboinfo[id].pos.snd + delta_y;
    roboinfo[id].pos = Pos(x,y);
    if (tile[y][x] == Base){
        if (id == my_id){
            // If moved to base, remove it from unvisited list:
            for (int i = 0; i < bases.size(); i++){
                if (bases[i] == Pos(x,y)){
                    bases[i] = bases.last();
                    bases.pop();
                    return; }
            }
        }
    }
}


// Let robot 'id' pick up package 'pkg_id'.
//
void Board::pick(int id, int pkg_id)
{
  #ifdef PARANOID
    for (int i = 0; i < roboinfo[id].packs.size(); i++) assert(roboinfo[id].packs[i] != pkg_id);
  #endif
    // Note that 'pkg_id' maybe larger than the known id's:
    packinfo.growTo(pkg_id+1);

    // Add package to robot's list of packages
    roboinfo[id].packs.push(pkg_id);

    // Set package position (to carried by robot):
    packinfo[pkg_id].pos = Pos(-1, id);
}


// Let robot 'id' drop package 'pkg_id'.
//
void Board::drop(int id, int pkg_id)
{
    Pos     pos = roboinfo[id].pos;

    // Note that 'pkg_id' maybe larger than the known id's (if I enter a continues game):
    packinfo.growTo(pkg_id+1);

    // Remove package from robot's list of packages
    vec<int>&   packs = roboinfo[id].packs;
    for (int i = 0; i < packs.size(); i++){
        if (packs[i] == pkg_id){
            packs[i] = packs.last();
            packs.pop();
            break; }
    }

    if (pos == packinfo[pkg_id].dest){
        if (id == my_id)
            score += packinfo[pkg_id].weight;
        packinfo[pkg_id].id = -1;
    }else{
        // Set package position:
        packinfo[pkg_id].pos = pos;
        // Set "dropped intentionally":
        packinfo[pkg_id].dropped = (id != my_id);   // If I dropped it, it's there (I know its destination)!
    }
}


// A robot enters the game -- or just an position update from server (not spec. but seems so)
//
void Board::enter(int id, int x, int y)
{
    roboinfo.growTo(id+1);
    roboinfo[id].id  = id;
    roboinfo[id].pos = Pos(x,y);
}


void Board::die(int id) {
    roboinfo[id].id = -1; }
