#include "Board.h"


//=================================================================================================
// Board dump method:


#define ESC "\x1B"

#define Land_attr ESC "[0;37;42;1m"
#define Aqua_attr ESC "[0;44m"
#define Wall_attr ESC "[0;30;41m"
#define Base_attr ESC "[0;31;42;1m"
#define Pkg_attr  ESC "[0;33;42;1m"
#define My_attr   ESC "[0;37;45;1m"
#define Foe_attr  ESC "[0;31;45;1m"
#define Reset     ESC "[0m"
#define Clear     ESC "[2J" ESC "[1;1H"
#define Home      ESC "[1;1H"
#define ClearLine ESC "[K"
#define Save      ESC "[s"
#define Restore   ESC "[u"


const char* tileAttr(Tile t) {
    switch (t){
    case Land: return Land_attr;
    case Aqua: return Aqua_attr;
    case Wall: return Wall_attr;
    case Base: return Base_attr;
    default: assert(false); } }

int tileChar(Tile t, bool deliv){
    switch (t){
    case Land: return deliv ? '$' : '.';
    case Aqua: return '~';
    case Wall: return '#';
    case Base: return '@';
    default: assert(false); } }


int carrying(Board& B);     // Temporary forward decl.

void Board::dump(bool ansi, bool big, bool clear)
{
    vec<vec<int> >  robot(height());      // 0=empty, 1=me, 2=other, 3=many
    for (int i = 0; i < height(); i++) robot[i].growTo(width(), 0);
    for (int i = 0; i < roboinfo.size(); i++){
        if (roboinfo[i].id == -1) continue;
        int x = roboinfo[i].pos.fst;
        int y = roboinfo[i].pos.snd;
        if (i == my_id || robot[y][x] == 1)
            robot[y][x] = 1;
        else{
            if (robot[y][x] == 0) robot[y][x] = 2;
            else                  robot[y][x] = 3;
        }
    }

    vec<vec<int> >  carried;
    vec<vec<int> >  packs(height());      // number of packages at a square. -1 means unvisited base or dropped package
    for (int i = 0; i < height(); i++) packs[i].growTo(width(), 0);
    for (int i = 0; i < packinfo.size(); i++){
        if (packinfo[i].id == -1) continue;
        int x = packinfo[i].pos.fst;
        int y = packinfo[i].pos.snd;
        if (x == -1) {
            carried.growTo(y+1);
            carried[y].push(i);
        }else{
            if (packs[y][x] > 0){
                if (!packinfo[i].dropped) packs[y][x]++;
            }else{
                if (!packinfo[i].dropped) packs[y][x] = 1;
                else                      packs[y][x] = -1;
            }
        }
    }
    for (int i = 0; i < bases.size(); i++){
        int x = bases[i].fst;
        int y = bases[i].snd;
        if (packs[y][x] == 0) packs[y][x] = -1;
    }

    vec<vec<char> >  mydeliv(height());      // 1=I have a delivery there
    for (int i = 0; i < height(); i++) mydeliv[i].growTo(width(), 0);
    for (int i = 0; i < me().packs.size(); i++){
        Package&    p = packinfo[me().packs[i]];
        if (!p.undeliverable)
            mydeliv[p.dest.snd][p.dest.fst] = 1;
    }

    static int dump_extra = -1;     // <= big hack warning
    if (ansi){
        if (!clear)
            printf(Reset);
        else{
            if (dump_extra == -1){
                dump_extra = 0;
                printf(Reset Clear);
            }else
                printf(Reset Home);
        }
    }

    if (big){
        printf("+"); for (int j = 0; j < tile.width()-2; j++) printf("---+"); printf("\n"); }

    for (int i = tile.height()-2; i >= 1; i--){
        vec<Tile>&     row = tile[i];

        if (!big){
            for (int j = 1; j < row.size()-1; j++){
                if      (robot[i][j] == 1) printf("%s", ansi ? My_attr  : "");
                else if (robot[i][j] == 2) printf("%sO", ansi ? Foe_attr : "");
                else if (robot[i][j] == 3) printf("%sX", ansi ? Foe_attr : "");
                else                       printf("%s%c", ansi ? tileAttr(row[j]) : "", tileChar(row[j], mydeliv[i][j]));
            }
            printf("%s\n", ansi ? Reset : "");

        }else{
            for (int line = 0; line < 2; line++){
                printf("|");
                for (int j = 1; j < row.size()-1; j++){
                    char    tmp[20] = "   "; tmp[0] = tmp[1] = tmp[2] = tileChar(row[j], mydeliv[i][j]);
                    if (line == 0){
                        if      (packs[i][j] == 0)   /*nothing*/;
                        else if (packs[i][j] == -1)  sprintf(tmp+2, "%s?" , ansi ? Pkg_attr : "");
                        else if (packs[i][j] < 10)   sprintf(tmp+2, "%s%d", ansi ? Pkg_attr : "", packs[i][j]);
                        else if (packs[i][j] < 100)  sprintf(tmp+1, "%s%d", ansi ? Pkg_attr : "", packs[i][j]);
                        else if (packs[i][j] < 1000) sprintf(tmp  , "%s%d", ansi ? Pkg_attr : "", packs[i][j]);
                        else                         sprintf(tmp  , "%sinf",ansi ? Pkg_attr : "");
                    }else if (line == 1){
                        if      (robot[i][j] == 1) sprintf(tmp+1, "%s><", ansi ? My_attr  : "");
                        else if (robot[i][j] == 2) sprintf(tmp+1, "%s<>", ansi ? Foe_attr : "");
                        else if (robot[i][j] == 3) sprintf(tmp+1, "%sXX", ansi ? Foe_attr : "");
                    }
                    printf("%s%s", ansi ? tileAttr(row[j]) : "", tmp);
                    printf("%s|", ansi ? Reset : "");
                }
                printf("\n");
            }
            printf("+"); for (int j = 0; j < row.size()-2; j++) printf("---+"); printf("\n");
        }
    }

    // Clear rest:
    if (ansi && clear){
        for (int i = 0; i < dump_extra+1; i++) printf("%s\n", ClearLine);
        int h = (!big) ? height()-2 : (height()-2)*3 + 1;
        printf(Home); for (int i = 0; i < h; i++) printf("\n");
    }

    dump_extra = 0;

    // Short status:
    printf("MyPos: (%d,%d)   Score: %d   Load: %d%%   Fuel: %d", me().pos.fst, me().pos.snd, score,
           100*carrying(*this) / capacity, money);
    if (!first_move){
        printf("   Cost: %d/%d\nMission: ", (int)mc.cost_so_far, (int)mc.estimated_cost); dump_extra++;
        mc.current.dump();dump_extra++; }

    // Packages: 
    printf("Carried packages:\n"); dump_extra++;
    for (int i = 0; i < carried.size(); i++){
        if (carried[i].size() > 0){
            printf(" %s%-3d:", (i == my_id) ? "*" : " ", i);
            for (int j = 0; j < carried[i].size(); j++){
                if (packinfo[carried[i][j]].id == -1)
                    printf(" %d", carried[i][j]);
                else
                    printf(" %d:%d(%d,%d)", carried[i][j], packinfo[carried[i][j]].weight, packinfo[carried[i][j]].dest.fst, packinfo[carried[i][j]].dest.snd);
            }
            printf("\n"); dump_extra++;
        }
    }

    // Top 10 missions:
    printf("\nMissions:\n"); dump_extra+=2;
    for (int i = 0; i < 10 && i < mc.last_missions.size(); i++){
        printf("  "); mc.last_missions[i].dump(); dump_extra++; }

    // Bans:
    printf("\nBan list:\n"); dump_extra+=2;
    vec<Pos> bans;
    mc.banned.domain(bans);
    for (int i = 0; i < bans.size(); i++){
        if (mc.banned.at(bans[i]) != 0)
            printf("  (%d,%d): %g\n", bans[i].fst, bans[i].snd, mc.banned.at(bans[i])); dump_extra++; }

    printf("-------------------------------------------------------------------------------\n");

    //**/if (mc.current.type == Harass) pause();

    // capacity, money left, capacity left, package lists
}
