#include "Heap.h"

/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/

float compare_HeapT(HeapT x, HeapT y) {
    return x.cost - y.cost; }

/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/

/* INTERNAL: 
*/

/* Macros uses free variables 'heap' and 'tmp': */

#define Parent(pos) ((pos+1)/2 - 1)
#define Left(pos)   ((pos)*2 + 1)
#define Right(pos)  ((pos)*2 + 2)
#define Valid(pos)  ((uint)pos < (uint)size())

#define Elem(pos)   array[pos]
#define Swap(x, y)  (tmp = Elem(x), Elem(x) = Elem(y), Elem(y) = tmp)


void Heap::siftDown(int pos)
{
    HeapT   tmp;
    int     smallest = pos;

    if (Valid(Left(pos))
     && compare_HeapT(Elem(smallest), Elem(Left(pos))) > 0)
        smallest = Left(pos);

    if (Valid(Right(pos))
     && compare_HeapT(Elem(smallest), Elem(Right(pos))) > 0)
        smallest = Right(pos);

    if (smallest != pos)
        Swap(pos, smallest),
        siftDown(smallest);
}

void Heap::siftUp(int pos)
{
    HeapT   tmp;

    if (Valid(Parent(pos))
     && compare_HeapT(Elem(pos), Elem(Parent(pos))) <= 0)
        Swap(pos, Parent(pos)),
        siftUp(Parent(pos));
}

#undef Swap
#undef Elem
#undef Valid
#undef Right
#undef Left
#undef Parent


/* PUBLIC:
*/


/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/

#if 0
void test(void)
{
    Heap    heap;
    int     i;

    heap.insert(1, Pos(0,0));
    heap.insert(2, Pos(0,0));
    heap.insert(3, Pos(0,0));
    heap.insert(4, Pos(0,0));
    heap.insert(5, Pos(0,0));

    for (i = 0; i < 100000; i++){
        if (heap.size() == 0 || ((rand() % 2)))
            heap.insert(rand() % 1000, Pos(0,0));
        else
            heap.pop();
    }

    while (heap.size() != 0) {
        printf("%d ", heap.peekCost());
        heap.pop(); }

    exit(0);
}
#endif
