#include <cstdio>
#include <cstdlib>
#include "Options.h"
#include "Heap.h"

#include "Sockets.h"
#include "ServerCom.h"

#define DUMP_MOVES_BRIEFLY


//=================================================================================================


#include "Mission.iC"       // (don't have time to do it properly with header files)


//-------------------------------------------------------------------------------------------------
// Move strategy:


// Simple strategy -- used for debugging against (when needing lots of players)
//
void genMove(Board& B)
{
    vec<int>    ps;     // packages here
    int         carrying = 0;
    for (int i = 0; i < B.packinfo.size(); i++){
        if (B.packinfo[i].id != -1){
            if (B.packinfo[i].pos == B.me().pos)
                ps.push(i);
            if (B.packinfo[i].pos == Pos(-1,B.my_id))
                carrying += B.packinfo[i].weight;
        }
    }

    vec<int>    pickup;
    for (int i = 0; i < ps.size(); i++){
        if (B.packinfo[ps[i]].weight + carrying <= B.capacity){
            pickup.push(ps[i]);
            carrying += B.packinfo[ps[i]].weight; }
    }

    if (pickup.size() > 0)
        performPick(B, 1/*bid*/, pickup);

    else if (B.me().packs.size() > 0){
        // Deliver package
        vec<int>    deliver;
        vec<int>&   qs = B.me().packs;
        for (int i = 0; i < qs.size(); i++){
            if (B.packinfo[qs[i]].dest == B.me().pos)
                deliver.push(qs[i]);
        }
        if (deliver.size() > 0)
            performDrop(B, 1/*bid*/, deliver);
        else{
            updateMatrices(B);
            performMove(B, 1/*bid*/, goToward(B, B.me().pos, B.packinfo[qs.last()].dest));
        }

    }else{
        // G inte bara mot baser! G mot ev. friliggande paket ocks.
        if (B.bases.size() == 0)
            printf("No unvisited bases, no packages left. Quitting.\n"), exit(0),
            performNop(B);
        else{
            updateMatrices(B);
            performMove(B, 1/*bid*/, goToward(B, B.me().pos, B.bases.last()));
        }
    }
}


bool waitReturn(void)
{
    char    buf[100];
    printf("[Press Return]"); fflush(stdout);
    fgets(buf, sizeof(buf), stdin);
    return (buf[0] == '\n');
}


void play(Board& B)
{
    int     fast_forward = -1;
    int     show_size    = 0;
    int     ansi         = 1;
    int     version      = 2;
    bool    hidden, big, clear;

    if (Options::value(Options::FF) != NULL)
        fast_forward = (Options::value(Options::FF)[0] == '\0') ? -1 : atoi(Options::value(Options::FF));
    if (Options::value(Options::SIZE) != NULL)
        show_size = atoi(Options::value(Options::SIZE));
    if (Options::value(Options::ANSI) != NULL)
        ansi = atoi(Options::value(Options::ANSI));
    if (Options::value(Options::VER) != NULL)
        version = atoi(Options::value(Options::VER));
    if (show_size == 0) hidden = true , big = false;
    else                hidden = false, big = (show_size == 2);
    clear = (ansi == 2);

    for (int c = 0;; c++){
        getPackages(B);
        if (!hidden) B.dump(ansi, big, clear);

        if (version == 1)
            genMove(B);
        else
            selectAndExecuteMove(B);

        if (!(fast_forward == -1 || c < fast_forward))
            if (!waitReturn()) break;
        getResult(B);
    }
}


//=================================================================================================


int main(int argc, char** argv)
{
    Board   B;

    try{
        Options::init(argc, argv);
    }catch (const char* msg){
        printf("%s\n", msg);
        exit(1);
    }
    if (Options::value(Options::HELP))
        fprintf(stderr,
        "USAGE: StNikolaus <host> <port> [<options>], where options are:\n"
        "\n"
        "  --ff=<num>    fast forward num turns (-1 = always)\n"
        "  --size=<num>  0=hidden, 1=small, 2=big (hidden default)\n"
        "  --ansi=<num>  0=off, 1=on, 2=special (on default)\n"
        "  --help=<num>  show this message\n\n"
        ), exit(0);

    initCommunication(argc, argv);
    getMap(B);
    play(B);

    return 0;
}
