#include "ServerCom.h"
#include "Sockets.h"
#include "Options.h"


//=================================================================================================
// Server replies/initialization:


void getMap(Board& B)
{
    vec<char*>  line;
    int         width, height;

    // Tell server I am here
    if (Options::value(Options::X) != NULL && Options::value(Options::Y))
        putLine(sFree(nsprintf("Login %s %s", Options::value(Options::X), Options::value(Options::Y))));
    else
        putLine("Player");
    getLine(line); assert(line.size() == 2);
    width  = atoi(line[0]);
    height = atoi(line[1]);
    B.tile.init(width+2, height+2);
    freeLine(line);

    // Draw frame 
    for (int j = 0; j < width+2 ; j++) B.tile[0][j] = B.tile[height+1][j] = Wall;
    for (int i = 1; i < height+1; i++) B.tile[i][0] = B.tile[i][width+1]  = Wall;

    // Read map
    B.initial_nof_bases  = 0; // (stats)
    for (int i = 1; i < height+1; i++){
        char*   tmp = getLine(); assert(tmp != NULL);
        for (int j = 1; j < width+1; j++){
            switch (tmp[j-1]){
            case '.': B.tile[i][j] = Land; break;
            case '#': B.tile[i][j] = Wall; break;
            case '~': B.tile[i][j] = Aqua; break;
            case '@': B.tile[i][j] = Base; B.bases.push(Pos(j,i));
                      B.initial_nof_bases++; break; // (stats)
            default: assert(false); }
        }
        xfree(tmp);
    }

    getLine(line); assert(line.size() == 3);
    B.my_id      = atoi(line[0]);
    B.capacity   = atoi(line[1]);
    B.money      = atoi(line[2]);
    B.score      = 0;
    B.first_move = true;
    freeLine(line);

    getLine(line);
    for (int i = 0; i < line.size(); i += 5){
        assert(line[i][0] == '#');
        int     id, x = -1, y = -1;
        id = atoi(&line[i][1]);
        if (line[i+1][0] == 'X') x = atoi(line[i+2]);
        else                     y = atoi(line[i+2]), assert(line[i+1][0] == 'Y');
        if (line[i+3][0] == 'X') x = atoi(line[i+4]);
        else                     y = atoi(line[i+4]), assert(line[i+3][0] == 'Y');
        assert(x != -1 && y != -1);

        B.enter(id, x, y);
    }
    freeLine(line);
}


void getPackages(Board& B)
{
    vec<char*>  line;

    getLine(line);
    if (line.size() > 0 && (line[0][0] < '0' || line[0][0] > '9')){
        printf("Strange server reply. Assuming game over.\nReply was:");
        for (int i = 0; i < line.size(); i++) printf(" %s", line[i]); printf("\n");
        printf("Died with score: %d\n", B.score);
        exit(1); }
    for (int i = 0; i < line.size(); i += 4){
        int     id      = atoi(line[i]);
        int     weight  = atoi(line[i+3]);
        Pos     dest    = Pos(atoi(line[i+1]), atoi(line[i+2]));
        B.packinfo.growTo(id+1);
        if (B.packinfo[id].id == -1){
            B.packinfo[id].id      = id;
            B.packinfo[id].weight  = weight;
            B.packinfo[id].pos     = B.me().pos;
            B.packinfo[id].dest    = dest;
            if (dest.fst < 0 || dest.fst >= B.width() || dest.snd < 0 || dest.snd >= B.height()){
                printf("Warning! Got package with destination outside map!\n");
                B.packinfo[id].undeliverable = true; }
        }else
            assert(B.packinfo[id].weight == weight && B.packinfo[id].dest == dest && B.packinfo[id].pos == B.me().pos);
        B.packinfo[id].dropped = false;
    }
    freeLine(line);
}


void getResult(Board& B)
{
    vec<char*>  line;
    int         id   = -1;
    vec<char>   alive;
    vec<char>   cmds;
    bool        pick_or_drop = false;

    getLine(line);
    for (int i = 0; i < B.roboinfo.size(); i++)
        B.roboinfo[i].last_move  = Idle,
        B.roboinfo[i].was_pushed = false;

    for (int i = 0; i < line.size();){
        if (line[i][0] == '#'){
            id = atoi(&line[i][1]);
            alive.growTo(id+1,0); alive[id] = 1;
            cmds .growTo(id+1,0);
            pick_or_drop = false;
            i++;
        }else{
            if (id == -1){
                printf("Strange server reply. Assuming game over.\nReply was:");
                for (int i = 0; i < line.size(); i++) printf(" %s", line[i]); printf("\n");
                printf("Died with score: %d\n", B.score);
                exit(1); }
            assert(line[i][1] == '\0');
            switch (line[i][0]){
            case 'N': B.move(id,  0, +1); i++; cmds[id]++; B.roboinfo[id].last_move = MoveN; break;
            case 'S': B.move(id,  0, -1); i++; cmds[id]++; B.roboinfo[id].last_move = MoveS; break;
            case 'E': B.move(id, +1,  0); i++; cmds[id]++; B.roboinfo[id].last_move = MoveE; break;
            case 'W': B.move(id, -1,  0); i++; cmds[id]++; B.roboinfo[id].last_move = MoveW; break;
            case 'P': B.pick(id, atoi(line[i+1])); i += 2; if (!pick_or_drop) cmds[id]++, pick_or_drop = true; B.roboinfo[id].last_move = Pick; break;
            case 'D': B.drop(id, atoi(line[i+1])); i += 2; if (!pick_or_drop) cmds[id]++, pick_or_drop = true; B.roboinfo[id].last_move = Drop; break;
            case 'X': assert(line[i+2][0] == 'Y'); B.enter(id, atoi(line[i+1]), atoi(line[i+3])); i += 4; break;
            default: assert(false); }
        }
    }
    freeLine(line);

    // See if robot died:
    alive.growTo(B.roboinfo.size(),0);
    cmds .growTo(B.roboinfo.size(),0);
    for (int i = 0; i < B.roboinfo.size(); i++)
        if (!alive[i] && B.roboinfo[i].id != -1)
            B.die(i);

    // See if robot was pushed: (= executed more than one command)
    for (int i = 0; i < B.roboinfo.size(); i++)
        if (B.roboinfo[i].id != -1 && cmds[i] > 1)
            B.roboinfo[i].was_pushed = true;        // (will not be set if didn't move first and had no package -- limitation in server protocol!)
}


//=================================================================================================
// Outgoing server commands:


macro void pushStr(vec<char>& list, char* text) {
    while (*text != '\0') list.push(*text++); }


void performNop(Board& B)
{
  #ifdef DUMP_MOVES_BRIEFLY
    pr(".");
  #endif
    putLine("1 Pick");
    B.money--;
    /**/pr("WARNING! No-op.\n");
}


static
void performPickOrDrop(Board& B, int bet, vec<int>& pickup, char* op)
{
    assert(bet != 0);
    vec<char>   text;
    pushStr(text, sFree(nsprintf("%d %s", bet, op)));
    for (int i = 0; i < pickup.size(); i++)
        pushStr(text, sFree(nsprintf(" %d", pickup[i])));
    text.push('\0');
    putLine(text);
    B.money -= abs(bet);
}


void performPick(Board& B, int bet, vec<int>& pickup) {
  #ifdef DUMP_MOVES_BRIEFLY
    pr("^");
  #endif
    performPickOrDrop(B, bet, pickup, "Pick"); }


void performDrop(Board& B, int bet, vec<int>& dropdown) {
  #ifdef DUMP_MOVES_BRIEFLY
    pr("v");
  #endif
    performPickOrDrop(B, bet, dropdown, "Drop"); }


void performMove(Board& B, int bet, int dir)
{
  #ifdef DUMP_MOVES_BRIEFLY
    pr((dir == North) ? "N" : (dir == South) ? "S" : (dir == East) ? "E" : (dir == West) ? "W" : "");
  #endif
    switch (dir){
    case North: putLine(sFree(nsprintf("%d Move N", bet))); B.money -= abs(bet); break;
    case South: putLine(sFree(nsprintf("%d Move S", bet))); B.money -= abs(bet); break;
    case East:  putLine(sFree(nsprintf("%d Move E", bet))); B.money -= abs(bet); break;
    case West:  putLine(sFree(nsprintf("%d Move W", bet))); B.money -= abs(bet); break;
    default:    performNop(B); break; }
}
