#include <unistd.h>
#include <netdb.h>
#include <sys/socket.h>
#include "Options.h"


static int socket_fd = -1;


static struct in_addr get_host_ipaddr(char* hostname)
{
    struct hostent* entry;

    entry = gethostbyname(hostname);
    if (entry == NULL)
        fprintf(stderr, "DNS lookup failed for: %s\n", hostname), exit(1);
    return *(struct in_addr*)entry->h_addr;
}


static int connect_socket(char* name, int port)
{
    int                 sock;
    struct sockaddr_in  addr;
    struct in_addr      iaddr;
    int                 ret;

    do{
        sock = socket(PF_INET, SOCK_STREAM, 0);
        if (sock < 0)
            perror("Call to 'socket()'"),
            exit(-1);

        iaddr = get_host_ipaddr(name);
        addr.sin_family = AF_INET;
        addr.sin_addr   = iaddr;
        addr.sin_port   = htons(port);

        ret = connect(sock, (struct sockaddr *)&addr, sizeof(struct sockaddr_in));
        if (ret >= 0)
            break;

        printf("Failed to connect, retrying...\n");
        close(sock);
        sleep(1);
    }while (1);

    return sock;
}


void initCommunication(int argc, char** argv)
{
    if (Options::nArgs() != 2)
    fprintf(stderr, "Please provide <server> <port> as arguments.\n"), exit(1);
    socket_fd = connect_socket(Options::arg(0), atoi(Options::arg(1)));
    printf("Connected successfully to server!\n");
}


// Send a line of text to server.
//
void putLine(char* text_)
{
    char*   text = xstrdup(text_);
    int     size = strlen(text);
    text[size] = '\n';
    size++;
    int     sent = 0;
    while (sent < size){
        int ret = write(socket_fd, text + sent, size - sent);
        if (ret <= 0)
            fprintf(stderr, "Lost connection with server during write!\n"), exit(1);
        else
            sent += ret;
    }
  #ifdef PARANOID
    pr("Sent line: %s\n", text_);
  #endif
    xfree(text);
}


// Get a line from server. Use 'xfree()' to dispose.
//
char* getLine(void)
{
    static vec<char>    buf;
    char    chr;
    int     ret;

    buf.clear();
    for(;;){
        ret = read(socket_fd, &chr, 1);
        if (ret <= 0)
            fprintf(stderr, "Lost connection with server during read!\n"), exit(1);
        if (chr == '\n')
            break;
        buf.push(chr);
    }
    buf.push('\0');

  #ifdef PARANOID
    pr("Received line: %s\n", (char*)buf);
  #endif
    return xstrdup(buf);
}


// Get a tokenized line from server. User 'freeLine()' to dispose.
//
void getLine(vec<char*>&  line)
{
    char*   tmp = getLine();
    int     i, j;

    if (tmp == NULL) return;

    for (i = 0; tmp[i] == ' '; i++);
    j = i;
    for (;;){
        if (tmp[j] == ' ' || tmp[j] == '\n' || tmp[j] == '\0'){
            if (i == j) break;
            char* tok = xmalloc<char>(j-i+1);
            tok[j-i] = '\0';
            memcpy(tok, &tmp[i], j-i);
            line.push(tok);
            for (i = j; tmp[i] == ' '; i++);
            j = i;
        }else
            j++;
    }
}


void freeLine(vec<char*>&  line)
{
    for (int i = 0; i < line.size(); i++)
        xfree(line[i]);
    line.clear();
}
